/**
 * Direct call of libjpeg to encode Jpeg from Inrimage
 * (c) Dominique Bereziat 2006
 */

#include <stdio.h>
#include <stdlib.h>
#include <jpeglib.h>
#include <inrimage/image.h>

/**
 * Limitations :
 *    - image NDIMZ convertie en DIMY
 *    - convertion si codage diffrent de 1 octet
 *    - colorspace limit a Grayscale et RGB
 *    - options cjpeg non implant (par flemme) : 
 *       -qtables, -qslots, -sample, -scans
 */

extern int debug_;

/* version : 1.1 is the old perl script version calling
 *           a patched version of cjpeg.
 *           1.2 was a fix on script perl adressing cjpeg options.
 *           2.0.0 is the new wrapper using directly libjpeg
 */

char version[] = "2.0.0";
char command[] = "[infile][jpeg-file][-C][cjpeg-like options]";
char details[] = "Encode an Inrimage image into Jpeg format.\n"
  "  -quality N: compression quality 0--100\n"
  "  -grayscale: create a grayscale image (incompatible with -C)\n"
  "   -optimize: optimize Huffman table (smaller file, but slow compression)\n"
  "-progressive: create a progressive JPEG image\n"
  " -dct int|fast|float : select the DCT method; float is the slower\n"
  "  -restart N: set restart interval in rows, or in blocks with B\n"
  "   -smooth N: smooth dithered input (N=1--100 is strength)\n"
  "   -baseline: force baseline output\n"
;

/**
 * retourne le nombre de lignes allouables (au plus dimy)
 * nelem : nombre d'lement par ligne
 * selem : taille d'un element
 */ 
int check_mem( size_t selem, size_t nelem, size_t dimy) {
  void *buf;
  
  do {
    buf = malloc( selem * nelem * dimy);
    if( buf) {
      free( buf);
      return dimy;
    }
    dimy /= 2;
  } while( dimy>1);
  return 1;
}

int main( int argc, char **argv) {
  /* libjpeg stuffs */
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  int quality = 75;
  JSAMPROW *row_pointer;
  int force_baseline = FALSE;
  int value;
  char ch = ' ';

  FILE *fp;
  /* inrimage stuffs */
  struct image *nf;
  Fort_int lfmt[9];
  Fort_int icodi[2], icodo[2], iexpi, iexpo;
  void *ibuf;
  float *fbuf;
  int i, ndimy_bl;

  char name[256];
  int direct_access;

  inr_init( argc, argv, version, command, details);
  infileopt( name);
  nf = image_( name, "e", " ", (NF_fmt*) lfmt);
  direct_access = ( TYPE == FIXE && BSIZE == 1);

  if( NDIMV == 2 || NDIMV > 3)
    imerror( 7, "format not supported");
  
  outfileopt(name);
  fp = *name=='>' ? stdout : fopen( name, "wb");
  if( fp == NULL) imerror( 1, "erreur ...");
  
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  jpeg_stdio_dest(&cinfo, fp);
  
  cinfo.image_width      = NDIMX;
  cinfo.image_height     = DIMY;
  cinfo.input_components = NDIMV;

  switch ( NDIMV) {
  case 1:
    cinfo.in_color_space = JCS_GRAYSCALE;
    break;
  case 3:
    cinfo.in_color_space = JCS_RGB;
    break;
  default:
    /* @FIXME */
    cinfo.in_color_space = JCS_UNKNOWN;
    break;
  }

  jpeg_set_defaults(&cinfo);

  /* libjpeg switches */
  force_baseline = igetopt0( "-baseline");
  if( igetopt1( "-quality", "%d", &quality))
    jpeg_set_quality(&cinfo, quality, force_baseline);
  igetopt1( "-smooth", "%d", &cinfo.smoothing_factor);
  cinfo.optimize_coding = igetopt0( "-optimize");
  if( igetopt0( "-progressive"))
    jpeg_simple_progression(&cinfo);
  if( igetopt1( "-restart", "%s", name) &&
      sscanf( name, "%ld%c", &value, &ch) >= 1) {
    if (ch == 'b' || ch == 'B') {
      cinfo.restart_interval = (unsigned int) value;
      cinfo.restart_in_rows = 0; 
    } else
      cinfo.restart_in_rows = (int) value;
  }
  if( igetopt1( "-dct", "%s", name)) {
    if( !strcmp( name, "int"))
      cinfo.dct_method = JDCT_ISLOW;
    else if( !strcmp( name, "fast"))
      cinfo.dct_method = JDCT_IFAST;
    else if( !strcmp( name, "float"))
      cinfo.dct_method = JDCT_FLOAT;
  }

  if( igetopt0( "-grayscale")) {
    cinfo.jpeg_color_space = JCS_GRAYSCALE;
    cinfo.num_components = 1;
  } else if( igetopt0( "-C")) { 
    cinfo.jpeg_color_space = JCS_RGB;
    cinfo.num_components = 3;
  }
  if( igetopt1( "-maxmemory", "%d", &value))
    cinfo.mem->max_memory_to_use = value * 1024;

  /* end switches */
  jpeg_start_compress(&cinfo, TRUE);

  /* allocate buffers */
  if( direct_access) {
    ndimy_bl = check_mem( sizeof(char), DIMX, DIMY);
    ibuf = (void*) i_malloc( DIMX*ndimy_bl);
  } else {
    ndimy_bl = check_mem( sizeof(float)+sizeof(char), DIMX, DIMY);
    ibuf = (void*) i_malloc( DIMX*ndimy_bl);
    fbuf = (void*) i_malloc( DIMX*sizeof(float)*ndimy_bl); 
    icodi[0] = sizeof(float);
    icodi[1] = REELLE;
    icodo[0] = 1;
    icodo[1] = FIXE;
    iexpi = iexpo = 0;
  }
  if( debug_) fprintf( stderr, "allocate bloc of %d lines\n", ndimy_bl);

  row_pointer = (JSAMPROW *)i_malloc( sizeof(JSAMPROW)*ndimy_bl);
  for( i=0; i<ndimy_bl; i++)
    row_pointer[i] = ibuf + i * DIMX;
  

  /* reading & writing */
  
  while (cinfo.next_scanline < cinfo.image_height) {
    int nlines;
    if( cinfo.next_scanline + ndimy_bl < cinfo.image_height)
      nlines = ndimy_bl;
    else
      nlines = cinfo.image_height - cinfo.next_scanline;    
    if( direct_access)
      c_lect( nf, nlines, ibuf);
    else {
      c_lecflt( nf, nlines, (void*)fbuf);
      c_cnvtbg( (void*)fbuf, (void*)ibuf, DIMX*nlines, icodi, icodo, iexpi, iexpo);
    }
    jpeg_write_scanlines(&cinfo, row_pointer, nlines);
  }

  jpeg_finish_compress(&cinfo);
  fclose( fp);
  fermnf_( &nf);
  jpeg_destroy_compress(&cinfo);
  i_Free( (void*)&ibuf);
  i_Free( (void*)&row_pointer);
  if( !direct_access)
    i_Free( (void*)&fbuf);
  return 0;
}
